class MSBController extends PlayerController
config (MSBCode);

var bool bHasInteraction;
var MSBInterAction MInteract;
var MSBMouseOverlay Mouse;

var() array<string> SaveGames;
var() array<string> SaveUTCharacters;
var() array<string> SaveCharacters;
var() array<string> SaveCharParts;
var() array<string> Save2DCharacters;
var() array<string> SaveProps;
var() array<string> SaveAnimations;
var() array<string> SavePoses;

var() config float MouseSens;
var() config array<string> VoxelShader;

var int screenX,
        screenY;

//Lists for quickly finding objects
var array<MSBGuiOverlay> MSBNodeButtons;
var array<MSBScriptNodeButton> LevelMSBScriptNodes;
var array<MSBNodeButton> LevelNodes;
var array<MSBExtendedListButton> LevelComplexButtons;
var () array<MSBActor> LevelActors;
var () array<MSBPawn> LevelPawns;
var array <Actor> LevelSpecial;


var bool bPlay;
var bool bMove;
var() config bool bGrid;
var() config int Gridspacing;

var float RunTime;

event PostBeginPlay()
{
local MSBGeneralPrefs gp;

super.PostBeginPlay();
gp=Level.game.LoadDataObject(class'MSBGeneralPrefs',"GeneralConfig","GeneralConfig");
if (gp==None)
  {
  Level.Game.CreateDataObject(class'MSBGeneralPrefs',"GeneralConfig","GeneralConfig");
  Level.Game.SavePackage("GeneralConfig");
  Log("Creating General Config File! Is this the first time you are running the program?");
  }
else
  LoadGeneralConfig(gp);
}

function PlayerTick(float DeltaTime)
{
super.PlayerTick(DeltaTime);
if (bPlay)
  RunTime+=DeltaTime;

if(Mouse!=None && !Mouse.bTyping)
  {
  Mouse.joyUaxis=aStrafe/100;
  Mouse.joyVaxis=aUp/100;
  Mouse.joyXaxis=(aTurn-aMouseX)/100;
  Mouse.joyYaxis=(-aLookUp+aMouseY)/100;
  }
}

state MSBCode
{
  function PlayerTick(float DeltaTime)
  {
  global.playerTick(DeltaTime);
  //2D Positioning
  if (Mouse!=None && !Mouse.bTyping)
   {
    Mouse.xPos+=DeltaTime*MouseSens*aMouseX+aStrafe/6000;
    Mouse.yPos-=DeltaTime*MouseSens*aMouseY+aForward/6000;
    }
  }

Begin:
 if (MInteract==None)
 {
   Player.InteractionMaster.AddInteraction("MSBCode.MSBInterAction", Player);
   bHasInteraction=true;
   Mouse=MSBHud(myHud).MouseOverlay;
   MInteract.Mouse=MSBHud(myHud).MouseOverlay;
   Log("PlayerController spawns an Interaction");
   }
}

state MoveObject
{

  function PlayerMove(float DeltaTime)
  {
        local vector X,Y,Z;

        ViewTarget.GetAxes(Rotation,X,Y,Z);
        Acceleration = 0.02 * (aForward*X + aStrafe*Y + aUp*vect(0,0,1));
        UpdateRotation(DeltaTime, 1);
        ViewTarget.MoveSmooth(SpectateSpeed * Normal(Acceleration) * DeltaTime);
        ViewTarget.SetRotation(Rotation);
  }
}

exec function RunMovie()
{
local MSBActor MSA;
local MSBPawn MSP;
local MSBInterpolationHelper I;
local MSBRootNode R;
local MSBScriptNode S;
local MSBScriptTriggerNode T;

bPlay=!bPlay;
Log("executing Start/Stop Script");

if (bPlay)          //start now!
  {
  Tag='running';
  RunTime=0;
  foreach AllActors(class'MSBRootNode',R)
    if (R!=None)
      R.initialize();

  foreach AllActors(class'MSBScriptTriggerNode',T)
    if (T!=None)
      T.initialize();

  foreach AllActors(class'MSBScriptNode',S)
    if (S!=None)
      S.initialize();


  foreach AllActors(class'MSBActor',MSA)
    if (MSA!=None)
      {
      MSA.RunMovie();
      //MSA.Reset();
      if (!MSA.bDisplayed)
        MSA.bHidden=true;
      }

  foreach AllActors(class'MSBPawn',MSP)
    if (MSP!=None)
      {
      MSP.iniPosing();
      MSP.RunMovie();
      }

  MSBHud(myHUD).DrawOverlays=false;
  }
else                 // end now!
  {
  Tag='constructing';

  foreach AllActors(class'MSBRootNode',R)
    if (R!=None)
      R.Reset();

  foreach AllActors(class'MSBScriptNode',S)
    if (S!=None)
      S.Reset();

  MSBHud(myHUD).DrawOverlays=true;

  foreach AllActors(class'MSBActor',MSA)
    if (MSA!=None)
      {
      MSA.bHidden=false;
      MSA.Reset();
      }
  foreach AllActors(class'MSBPawn',MSP)
    if (MSP!=None)
        MSP.clearPosing();

  foreach AllActors(class'MSBInterpolationHelper',I)
    {
    Log("Destroying:"@i);
    if (I!=None)
      I.Destroy();
    }

  }
}

exec function switchStates()
{
bMove=!bMove;
if (bMove)
  GotoState('Spectating');
else
  GotoState('MSBCode');
}

function calculateGrid(out vector gridPos)
{
gridPos.X=int(gridPos.X/gridSpacing)*gridSpacing;
gridPos.Y=int(gridPos.Y/gridSpacing)*gridSpacing;
gridPos.Z=int(gridPos.Z/gridSpacing)*gridSpacing;
}

exec function ToggleGrid()
{
bGrid=!bGrid;
}

exec function SetGridSpacing(int spacing)
{
GridSpacing=spacing;
}


exec function LoadGeneralConfig(MSBGeneralPrefs gp)
{
local int i;
for (i=0;i<gp.saveGames.length;i++)
  Savegames[i]=gp.SaveGames[i];

for (i=0;i<gp.saveUTCharacters.length;i++)
  SaveUTCharacters[i]=gp.SaveUTCharacters[i];

for (i=0;i<gp.saveCharacters.length;i++)
  {
  SaveCharacters[i]=gp.SaveCharacters[i];
  }

for (i=0;i<gp.saveCharParts.length;i++)
  SaveCharParts[i]=gp.SaveCharParts[i];

for (i=0;i<gp.save2DCharacters.length;i++)
  Save2DCharacters[i]=gp.Save2DCharacters[i];

for (i=0;i<gp.saveProps.length;i++)
  SaveProps[i]=gp.SaveProps[i];

for (i=0;i<gp.saveAnimations.length;i++)
  SaveAnimations[i]=gp.SaveAnimations[i];

for (i=0;i<gp.savePoses.length;i++)
  SavePoses[i]=gp.SavePoses[i];
Log("General Config loaded");
}

exec function SaveGeneralConfig()
{
local int i;
local MSBGeneralPrefs gp;

gp=Level.Game.LoadDataObject(class'MSBGeneralPrefs',"GeneralConfig","GeneralConfig");
for (i=0;i<saveGames.length;i++)
  gp.SaveGames[i]=Savegames[i];

for (i=0;i<saveUTCharacters.length;i++)
  gp.SaveUTCharacters[i]=SaveUTCharacters[i];

for (i=0;i<saveCharacters.length;i++)
  gp.SaveCharacters[i]=SaveCharacters[i];

for (i=0;i<saveCharParts.length;i++)
  gp.SaveCharParts[i]=SaveCharParts[i];

for (i=0;i<save2DCharacters.length;i++)
  gp.Save2DCharacters[i]=Save2DCharacters[i];

for (i=0;i<saveProps.length;i++)
  gp.SaveProps[i]=SaveProps[i];

for (i=0;i<saveAnimations.length;i++)
  gp.SaveAnimations[i]=SaveAnimations[i];

for (i=0;i<savePoses.length;i++)
  gp.SavePoses[i]=SavePoses[i];
Level.Game.SavePackage("GeneralConfig");
Log("General Config written!");
}

exec function CleanUp()
{
local int i;
local ScriptedSequence S;

//remove Interaction
MInteract.M=None;
MInteract.Mouse=None;
Player.InteractionMaster.RemoveInteraction(MInteract);
Log("removing Interaction"@MInteract);
MInteract=None;
bHasInteraction=false;
MSBHud(myHud).DoFOverlay.Destroy();
//destroy all actions
foreach AllActors(class'ScriptedSequence', S)
  for (i=0;i<S.Actions.length;i++)
    S.Actions[i]=None;
// clear MSBNodeButton array
for (i=0;i<MSBNodeButtons.length;i++)
  MSBNodeButtons[i]=None;
}


// ********************************************************************************
// general Commands and "soon to be implemented" things
// ********************************************************************************

exec function placeMSBActor(string SpawnActorClassName)
{
local MSBActor SpawnActor;
local vector x,y,z;
local class <MSBActor> SpawnActorClass;

if (SpawnActorClass==None)
  SpawnActorClass = class<MSBActor>(DynamicLoadObject(SpawnActorClassName, class'Class'));
SpawnActor = Spawn(SpawnActorClass,,,Location,Rotation);

LevelActors.Insert(LevelActors.Length,1);
LevelActors[LevelActors.Length-1]=SpawnActor;
GetAxes(Rotation,x,y,z);
SetLocation(SpawnActor.Location-X*100);
}

exec function importPose(string PoseName, string PosePackagename)
{
local int i;
local MSBAniKey A,B;
local bool bOverwrite;
Log("loading Pose!");

for (i=0;i<63;i++)
  {
  A=Level.Game.LoadDataObject(class'MSBAniKey',PoseName$"Key"$i,PosePackageName);
  if (A!=None)
    {
    B=Level.Game.CreateDataObject(class'MSBAniKey',PoseName$"Key"$i,"MovieSandboxPoses");
    B.BoneRotation=A.BoneRotation;
    B.BoneNumber=A.BoneNumber;
    }
  }

//update SavePose List
for (i=0;i<SavePoses.length;i++)
  if (PoseName==SavePoses[i])
     bOverwrite=true;
if (!bOverwrite)
  SavePoses[SavePoses.Length]=PoseName;
SaveGeneralConfig();
}

exec function SetBrushSize(float size)
{
local MSBSimpleBlock block;

foreach allactors (class'MSBsimpleBlock',block)
  block.SetDrawScale(size);
}

exec function BlocksToBrushes()
{
local MSBSimpleBlock block;

foreach allactors (class'MSBsimpleBlock',block)
  {
   if (block.Shader==None)
     {
     block.Shader=new class'MSBBrushShader';
     block.shader.Parent=block;
     Log(block.shader);
     }
  }
}

defaultproperties
{
InitialState=MSBCode
bHasInteraction=true
MouseSens=0.7
bGrid=true
Gridspacing=16
SpectateSpeed=100
}
